@@ -266,6 +266,9 @@ int client_main(int argc, char **argv)
     }
   }
 
+  /* start vncserver */
+  StartVNCServer();
+
   if (auto_flag == 0) {
     mini_window = (GtkWidget *)create_mini_window();
     gtk_widget_show(mini_window);
@@ -352,6 +355,7 @@ void
 JoinVNC () {
   char commandvnc1[256] = "vncconfig -display :";
   char commandvnc2[256] = "vncconnect -display :";
+  char commandvnc3[256] = "vnc4config -display :";
   char serverport[10];
   int errno;
   
@@ -359,22 +363,29 @@ JoinVNC () {
   fprintf(stderr, "vncdisplay=%s\n",vncdisplay);
   strcat (commandvnc1, vncdisplay);
   strcat (commandvnc2, vncdisplay);
+  strcat (commandvnc3, vncdisplay);
   strcat (commandvnc1, " -connect ");
   strcat (commandvnc2, " ");
+  strcat (commandvnc3, " -connect ");
   strcat (commandvnc1, (char *)inet_ntoa(client_CB.server_ip));
   strcat (commandvnc2, (char *)inet_ntoa(client_CB.server_ip));
+  strcat (commandvnc3, (char *)inet_ntoa(client_CB.server_ip));
   strcat (commandvnc1, ":");
   strcat (commandvnc2, ":");
+  strcat (commandvnc3, ":");
   sprintf (serverport, "%d\0", ntohs(client_CB.server_port));
   strcat (commandvnc1, serverport);
   strcat (commandvnc2, serverport);
+  strcat (commandvnc3, serverport);
   fflush (stdout);
   fprintf (stdout, "%s\n", commandvnc1);
   fprintf (stdout, "%s\n", commandvnc2);
+  fprintf (stdout, "%s\n", commandvnc3);
 
   /* exeute vncconnect */
   client_CB.vncdsp = system(commandvnc1);
   client_CB.vncdsp = system(commandvnc2);
+  client_CB.vncdsp = system(commandvnc3);
 
   client_CB.connectstate = STAT_VNC;
 }
@@ -554,7 +565,8 @@ GetMultiVNCServerMsg (void *g_dsp)
     //gdk_input_remove(client_CB.g_ctldsp);
     bzero (buf, 2);
     if (read (client_CB.ctldsp, buf, 2) <= 0) {
-      fprintf(stderr, "%s : error connection\n", programName);
+      CloseVNCServer();
+      fprintf(stderr, "%s : session closed\n", programName);
       exit (1);
     }
     printf("type=%d, flag=%d\n",buf[0],buf[1]);
@@ -576,7 +588,8 @@ GetMultiVNCServerMsg (void *g_dsp)
 
 	printf("switch : rfbCastAdv=%d\n",GETRECV_C);
 	if (read (client_CB.ctldsp, (char *)&castadv + 2, sizeof(CastAdv)-2) <= 0) {
-	  fprintf(stderr, "%s : error connection\n", programName);
+	  CloseVNCServer();
+	  fprintf(stderr, "%s : session closed\n", programName);
 	  exit (1);
 	}
 
@@ -623,7 +636,8 @@ GetMultiVNCServerMsg (void *g_dsp)
 	
 	printf("switch : rfbLockAdv\n");
 	if (read (client_CB.ctldsp, (char *)&lockadv + 2, sizeof(LockAdv)-2) <= 0) {
-	  fprintf(stderr, "%s : error connection\n", programName);
+	  CloseVNCServer();
+	  fprintf(stderr, "%s : session closed\n", programName);
 	  exit (1);
 	}
 
@@ -1009,3 +1023,29 @@ help (char *cmd)
   fprintf(stderr, "  -u [user_name]:Input client id and auto connect.\n\n");
 }
 
+void
+StartVNCServer()
+{
+  char vncserver_cmd[16] = "vncserver :";
+  if (atoi(vncdisplay) != 0) {
+    printf("---start vncserver---\n");
+    strcat(vncserver_cmd, vncdisplay);
+    printf("%s", vncserver_cmd);
+    system(vncserver_cmd);
+    printf("\n-----\n\n");
+  }
+}
+
+void
+CloseVNCServer()
+{
+  char vncserver_cmd[32] = "vncserver -kill :";
+  if (atoi(vncdisplay) != 0) {
+    printf("---close vncserver---\n");
+    strcat(vncserver_cmd, vncdisplay);
+    printf("%s", vncserver_cmd);
+    system(vncserver_cmd);
+    printf("\n-----\n\n");
+  }
+}
+
